"""App for managing character standing requests, made for Alliance Auth."""

# pylint: disable = invalid-name

default_app_config = "standingsrequests.apps.StandingsRequestsConfig"

__version__ = "2.0.0"
__title__ = "Standings Requests"
