# list of entity type IDs used in unit tests

from standingsrequests.core.contact_types import ContactTypeId

CHARACTER_TYPE_ID = ContactTypeId.CHARACTER_AMARR

CORPORATION_TYPE_ID = 2

ALLIANCE_TYPE_ID = 16159

FACTION_CALDARI_STATE_TYPE_ID = 500001

FACTION_TYPE_ID = FACTION_CALDARI_STATE_TYPE_ID
