{% extends 'allianceauth/base-bs5.html' %}
{% load i18n %}
{% load static %}

{% block header_nav_brand %}{% translate "Standing Requests" %}{% endblock %}

{% block page_title %}{% translate app_title %}{% endblock %}

{% block content %}
{% endblock %}

{% block header_nav_collapse_left %}
    {% include 'standingsrequests/partials/menu.html' %}
{% endblock %}

{% block header_nav_collapse_right %}
    {% include 'standingsrequests/partials/right-menu.html' %}
{% endblock %}

{% block extra_css %}
    <link rel="stylesheet" href="{% static 'standingsrequests/css/global.css' %}" type="text/css" media="screen">
{% endblock extra_css %}

{% block header_nav_user_character_control %}
    {% comment %}
        This block is left empty intentionally to disable defaults.
    {% endcomment %}
{% endblock %}
