{% load i18n %}
{% load navactive %}

<li class="nav-item">
    <a class="nav-link {% navactive request 'standingsrequests:create_requests' %}" href="{% url 'standingsrequests:create_requests' %}">{% translate "My Requests" %}</a>
</li>

{% if perms.standingsrequests.affect_standings %}
    <li class="nav-item">
        <a class="nav-link {% navactive request 'standingsrequests:manage' %}" href="{% url 'standingsrequests:manage' %}">
            {% translate "Manage Requests" %}
            {% if pending_total_count > 0 %}
                &nbsp;<span class="badge badge-nav">{{ pending_total_count }}</span>
            {% endif %}
        </a>
    </li>
{% endif %}

<li class="nav-item">
    <a class="nav-link {% navactive request 'standingsrequests:standings' %}" href="{% url 'standingsrequests:standings' %}">
        {{ operation_mode|capfirst }} {% translate "Standings" %}
    </a>
</li>
